@@ -14,7 +14,20 @@ import pytest
 pytest_plugins = ("pytester",)
 
 
+def handle_tr_writer_deprecation():
+    # Remove this function when they've fixed
+    # https://github.com/pytest-dev/pytest/issues/6936
+    import warnings
+    from _pytest.warnings import _setoption
+
+    arg = "ignore:TerminalReporter.writer:pytest.PytestDeprecationWarning"
+    _setoption(warnings, arg)
+
+
 def run(testdir, path="report.html", *args):
+    # TODO: Temporary hack until they fix
+    # https://github.com/pytest-dev/pytest/issues/6936
+    handle_tr_writer_deprecation()  # TODO: Temporary hack
     path = testdir.tmpdir.join(path)
     result = testdir.runpytest("--html", path, *args)
     return result, read_html(path)
@@ -219,6 +232,9 @@ class TestHTML:
         assert report_title in html
 
     def test_report_title_addopts_env_var(self, testdir, monkeypatch):
+        # TODO: Temporary hack until they fix
+        # https://github.com/pytest-dev/pytest/issues/6936
+        handle_tr_writer_deprecation()
         report_location = "REPORT_LOCATION"
         report_name = "MuhReport"
         monkeypatch.setenv(report_location, report_name)
@@ -418,6 +434,35 @@ class TestHTML:
         self.test_extra_image(testdir, "image/png", "png")
         assert mock_isfile.call_count == 1
 
+    @pytest.mark.parametrize("mime_type, extension", [("video/mp4", "mp4")])
+    def test_extra_video(self, testdir, mime_type, extension):
+        content = str(random.random())
+        testdir.makeconftest(
+            f"""
+            import pytest
+            @pytest.hookimpl(hookwrapper=True)
+            def pytest_runtest_makereport(item, call):
+                outcome = yield
+                report = outcome.get_result()
+                if report.when == 'call':
+                    from pytest_html import extras
+                    report.extra = [extras.{extension}('{content}')]
+        """
+        )
+        testdir.makepyfile("def test_pass(): pass")
+        result, html = run(testdir, "report.html", "--self-contained-html")
+        assert result.ret == 0
+        src = f"data:{mime_type};base64,{content}"
+        assert (
+            f'<video controls><source src="{src}" type="{mime_type}"></video>' in html
+        )
+
+    def test_extra_video_windows(self, mocker, testdir):
+        mock_isfile = mocker.patch("pytest_html.plugin.isfile")
+        mock_isfile.side_effect = ValueError("stat: path too long for Windows")
+        self.test_extra_video(testdir, "video/mp4", "mp4")
+        assert mock_isfile.call_count == 1
+
     @pytest.mark.parametrize(
         "content", [("u'\u0081'"), ("'foo'"), ("b'\\xe2\\x80\\x93'")]
     )
@@ -554,6 +599,20 @@ class TestHTML:
         assert link in html
         assert os.path.exists(src)
 
+    def test_extra_fixture(self, testdir):
+        content = b64encode(b"foo").decode("ascii")
+        testdir.makepyfile(
+            f"""
+            def test_pass(extra):
+                from pytest_html import extras
+                extra.append(extras.png('{content}'))
+        """
+        )
+        result, html = run(testdir, "report.html", "--self-contained-html")
+        assert result.ret == 0
+        src = f"data:image/png;base64,{content}"
+        assert f'<img src="{src}"/>' in html
+
     def test_no_invalid_characters_in_filename(self, testdir):
         testdir.makeconftest(
             """
@@ -633,10 +692,22 @@ class TestHTML:
         assert "Environment" in html
         assert len(re.findall(content, html)) == 1
 
-    def test_environment_list_value(self, testdir):
-        content = tuple(str(random.random()) for i in range(10))
-        content += tuple(random.random() for i in range(10))
-        expected_content = ", ".join(str(i) for i in content)
+    _unsorted_tuples = [
+        ("Hello", "fzWZP6vKRv", "hello", "garAge", "123Go"),
+        (2, 4, 2, 1, 54),
+        ("Yes", 400, "5.4"),
+    ]
+    _sorted_tuples = [
+        "123Go, Hello, fzWZP6vKRv, garAge, hello",
+        "1, 2, 2, 4, 54",
+        "400, 5.4, Yes",
+    ]
+    _test_environment_list_value_data_set = zip(_unsorted_tuples, _sorted_tuples)
+
+    @pytest.mark.parametrize(
+        "content,expected_content", _test_environment_list_value_data_set
+    )
+    def test_environment_list_value(self, testdir, content, expected_content):
         expected_html_re = fr"<td>content</td>\n\s+<td>{expected_content}</td>"
         testdir.makeconftest(
             f"""
@@ -651,6 +722,46 @@ class TestHTML:
         assert "Environment" in html
         assert len(re.findall(expected_html_re, html)) == 1
 
+    _unordered_dict = {k: len(k) for k in _unsorted_tuples[0]}
+    _unordered_dict_expected = (
+        r'<td>content</td>\n\s+<td>{"123Go": 5, "Hello": 5, '
+        r'"fzWZP6vKRv": 10, "garAge": 6, "hello": 5}</td>'
+    )
+    _unordered_dict_with_html = {
+        "First Link": r'<a href="https://www.w3schools.com">W3Schools</a>',
+        "Second Link": r'<a href="https://www.w3schools.com">W2Schools</a>',
+        "Third Link": r'<a href="https://www.w3schools.com">W4Schools</a>',
+    }
+    _unordered_dict_with_html_expected = (
+        r"<td>content</td>\n\s+<td>{"
+        r'"First Link": "<a href=\\"https://www.w3schools.com\\">W3Schools</a>", '
+        r'"Second Link": "<a href=\\"https://www.w3schools.com\\">W2Schools</a>", '
+        r'"Third Link": "<a href=\\"https://www.w3schools.com\\">W4Schools</a>"}</td>'
+    )
+
+    @pytest.mark.parametrize(
+        "unordered_dict,expected_output",
+        [
+            (_unordered_dict, _unordered_dict_expected),
+            (_unordered_dict_with_html, _unordered_dict_with_html_expected),
+        ],
+    )
+    def test_environment_unordered_dict_value(
+        self, testdir, unordered_dict, expected_output
+    ):
+        testdir.makeconftest(
+            f"""
+            def pytest_configure(config):
+                values = dict({json.dumps(unordered_dict)})
+                config._metadata['content'] = values
+        """
+        )
+        testdir.makepyfile("def test_pass(): pass")
+        result, html = run(testdir)
+        assert result.ret == 0
+        assert "Environment" in html
+        assert len(re.findall(expected_output, html)) == 1
+
     def test_environment_ordered(self, testdir):
         testdir.makeconftest(
             """
@@ -783,6 +894,9 @@ class TestHTML:
         assert "No such file or directory: 'style.css'" in result.stderr.str()
 
     def test_css_invalid_no_html(self, testdir):
+        # TODO: Temporary hack until they fix
+        # https://github.com/pytest-dev/pytest/issues/6936
+        handle_tr_writer_deprecation()
         testdir.makepyfile("def test_pass(): pass")
         result = testdir.runpytest("--css", "style.css")
         assert result.ret == 0
@@ -799,3 +913,43 @@ class TestHTML:
         result, html = run(testdir)
         assert result.ret == 0
         assert r"\u6d4b\u8bd5\u7528\u4f8b\u540d\u79f0" not in html
+
+    @pytest.mark.parametrize("is_collapsed", [True, False])
+    def test_collapsed(self, testdir, is_collapsed):
+        collapsed_html = '<tr class="collapsed">'
+        expected_count = 2 if is_collapsed else 0
+        testdir.makeini(
+            f"""
+            [pytest]
+            render_collapsed = {is_collapsed}
+        """
+        )
+        testdir.makepyfile(
+            """
+            def test_fail():
+                assert False
+
+            def test_pass():
+                assert True
+        """
+        )
+        result, html = run(testdir)
+        assert result.ret == 1
+        assert len(re.findall(collapsed_html, html)) == expected_count
+        assert_results(html, tests=2, passed=1, failed=1)
+
+    def test_custom_content_report_title(self, testdir):
+        content_report_title = str(random.random())
+        testdir.makeconftest(
+            f"""
+            import pytest
+            from py.xml import html
+
+            def pytest_html_report_title(report):
+                report.title = "title is {content_report_title}"
+        """
+        )
+        testdir.makepyfile("def test_pass(): pass")
+        result, html = run(testdir)
+        assert result.ret == 0
+        assert len(re.findall(content_report_title, html)) == 1
